'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _commentJson;

function _load_commentJson() {
  return _commentJson = _interopRequireDefault(require('comment-json'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../logger/logger'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _componentMap;

function _load_componentMap() {
  return _componentMap = _interopRequireDefault(require('./component-map'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var BitMap = function () {
  // path => componentId

  function BitMap(projectRoot, mapPath, components, version) {
    _classCallCheck(this, BitMap);

    this.projectRoot = projectRoot;
    this.mapPath = mapPath;
    this.components = components;
    this.hasChanged = false;
    this.version = version;
    this.paths = {};
  }

  _createClass(BitMap, [{
    key: 'getAllComponents',
    value: function getAllComponents(origin) {
      if (!origin) return this.components;
      var isOriginMatch = function isOriginMatch(component) {
        return component.origin === origin;
      };
      var isOriginMatchArray = function isOriginMatchArray(component) {
        return origin.includes(component.origin);
      };
      var filter = Array.isArray(origin) ? isOriginMatchArray : isOriginMatch;
      return (_ramda || _load_ramda()).default.filter(filter, this.components);
    }
  }, {
    key: 'getAuthoredExportedComponents',
    value: function getAuthoredExportedComponents() {
      var _this = this;

      var componentsIds = [];
      Object.keys(this.components).forEach(function (componentId) {
        if (_this.components[componentId].origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) {
          var idParsed = (_bitId || _load_bitId()).BitId.parse(componentId);
          if (idParsed.scope) componentsIds.push(idParsed);
        }
      });
      return componentsIds;
    }
  }, {
    key: '_makePathRelativeToProjectRoot',
    value: function _makePathRelativeToProjectRoot(pathToChange) {
      var absolutePath = (_path || _load_path()).default.resolve(pathToChange);
      return (_path || _load_path()).default.relative(this.projectRoot, absolutePath);
    }

    // todo - need to move to bit-javascript

  }, {
    key: '_searchMainFile',
    value: function _searchMainFile(baseMainFile, files, originalMainFile) {
      var newBaseMainFile = void 0;
      // Search the relativePath of the main file
      var mainFileFromFiles = (_ramda || _load_ramda()).default.find((_ramda || _load_ramda()).default.propEq('relativePath', baseMainFile))(files);
      // Search the base name of the main file and transfer to relativePath
      if ((_ramda || _load_ramda()).default.isNil(mainFileFromFiles)) {
        var potentialMainFiles = files.filter(function (file) {
          return file.name === baseMainFile;
        });
        if (potentialMainFiles.length) {
          // when there are several files that met the criteria, choose the closer to the root
          var sortByNumOfDirs = function sortByNumOfDirs(a, b) {
            return a.relativePath.split((_constants || _load_constants()).DEFAULT_SEPARATOR).length - b.relativePath.split((_constants || _load_constants()).DEFAULT_SEPARATOR).length;
          };
          potentialMainFiles.sort(sortByNumOfDirs);
          mainFileFromFiles = (_ramda || _load_ramda()).default.head(potentialMainFiles);
        }
        newBaseMainFile = mainFileFromFiles ? mainFileFromFiles.relativePath : baseMainFile;
        return { mainFileFromFiles, baseMainFile: newBaseMainFile || baseMainFile };
      }
      return { mainFileFromFiles, baseMainFile: originalMainFile };
    }
  }, {
    key: '_getMainFile',
    value: function _getMainFile(mainFile, componentMap) {
      var _this2 = this;

      var files = componentMap.files.filter(function (file) {
        return !file.test;
      });
      // Take the file path as main in case there is only one file
      if (!mainFile && files.length === 1) return files[0].relativePath;
      // search main file (index.js or index.ts) in case no main file was entered - move to bit-javascript
      var searchResult = this._searchMainFile(mainFile, files, mainFile);
      if (!searchResult.mainFileFromFiles) {
        // TODO: can be improved - stop loop if finding main file
        (_constants || _load_constants()).DEFAULT_INDEX_EXTS.forEach(function (ext) {
          if (!searchResult.mainFileFromFiles) {
            var mainFileNameToSearch = `${(_constants || _load_constants()).DEFAULT_INDEX_NAME}.${ext}`;
            searchResult = _this2._searchMainFile(mainFileNameToSearch, files, mainFile);
          }
        });
      }

      // When there is more then one file and the main file not found there
      if ((_ramda || _load_ramda()).default.isNil(searchResult.mainFileFromFiles)) {
        var mainFileString = mainFile || `${(_constants || _load_constants()).DEFAULT_INDEX_NAME}.[${(_constants || _load_constants()).DEFAULT_INDEX_EXTS.join(', ')}]`;
        throw new (_exceptions || _load_exceptions()).MissingMainFile(mainFileString, files.map(function (file) {
          return (_path || _load_path()).default.normalize(file.relativePath);
        }));
      }
      return searchResult.baseMainFile;
    }
  }, {
    key: 'addDependencyToParent',
    value: function addDependencyToParent(parent, dependency) {
      // the parent component might appear in bit.map file with full-id, e.g. when it is a dependency itself.
      // And it might be with the short id, without the scope and the version, e.g. when its origin is AUTHORED or IMPORTED
      var parentWithScope = parent.toString();
      var parentWithoutScope = parent.changeScope(null).toString();
      var parentId = void 0;
      if (this.components[parentWithScope]) {
        parentId = parentWithScope;
      } else if (this.components[parentWithoutScope]) {
        parentId = parentWithoutScope;
      } else {
        throw new Error(`Unable to add indirect dependency ${dependency}, as its parent ${parent} does not exist`);
      }
      if (!this.components[parentId].dependencies) {
        this.components[parentId].dependencies = [dependency];
      }
      if (!this.components[parentId].dependencies.includes(dependency)) {
        this.components[parentId].dependencies.push(dependency);
      }
    }
  }, {
    key: 'deleteOlderVersionsOfComponent',
    value: function deleteOlderVersionsOfComponent(componentId) {
      var _this3 = this;

      var allVersions = Object.keys(this.components).filter(function (id) {
        return (_bitId || _load_bitId()).BitId.parse(id).toStringWithoutVersion() === componentId.toStringWithoutVersion();
      });
      allVersions.forEach(function (version) {
        if (version !== componentId.toString() && _this3.components[version].origin !== (_constants || _load_constants()).COMPONENT_ORIGINS.NESTED) {
          (_logger || _load_logger()).default.debug(`BitMap: deleting an older version ${version} of an existing component ${componentId.toString()}`);
          _this3._removeFromComponentsArray(version);
        }
      });
    }

    /**
     * When the given id doesn't include scope-name, there might be a similar component in bit.map with scope-name
     */

  }, {
    key: 'getExistingComponentId',
    value: function getExistingComponentId(componentIdStr) {
      if (this.components[componentIdStr]) return componentIdStr;
      var parsedId = (_bitId || _load_bitId()).BitId.parse(componentIdStr);
      if (parsedId.scope && !parsedId.hasVersion()) {
        return Object.keys(this.components).find(function (component) {
          return (_bitId || _load_bitId()).BitId.parse(component).toStringWithoutVersion() === componentIdStr;
        });
      }
      return Object.keys(this.components).find(function (component) {
        return (_bitId || _load_bitId()).BitId.parse(component).toStringWithoutScopeAndVersion() === componentIdStr;
      });
    }
  }, {
    key: 'addComponent',
    value: function addComponent(_ref) {
      var componentId = _ref.componentId,
          files = _ref.files,
          mainFile = _ref.mainFile,
          origin = _ref.origin,
          parent = _ref.parent,
          rootDir = _ref.rootDir,
          override = _ref.override,
          originallySharedDir = _ref.originallySharedDir;

      var isDependency = origin === (_constants || _load_constants()).COMPONENT_ORIGINS.NESTED;
      var componentIdStr = componentId.toString();
      (_logger || _load_logger()).default.debug(`adding to bit.map ${componentIdStr}`);
      if (isDependency) {
        if (!parent) throw new Error(`Unable to add indirect dependency ${componentIdStr}, without "parent" parameter`);
        this.addDependencyToParent(parent, componentIdStr);
      }
      if (this.components[componentIdStr]) {
        (_logger || _load_logger()).default.info(`bit.map: updating an exiting component ${componentIdStr}`);
        var existingRootDir = this.components[componentIdStr].rootDir;
        if (existingRootDir) (_componentMap || _load_componentMap()).default.changeFilesPathAccordingToItsRootDir(existingRootDir, files);
        if (override) {
          this.components[componentIdStr].files = files;
        } else {
          this.components[componentIdStr].files = (_ramda || _load_ramda()).default.unionWith((_ramda || _load_ramda()).default.eqBy((_ramda || _load_ramda()).default.prop('relativePath')), files, this.components[componentIdStr].files);
        }
        if (mainFile) {
          this.components[componentIdStr].mainFile = this._getMainFile((0, (_utils || _load_utils()).pathNormalizeToLinux)(mainFile), this.components[componentIdStr]);
        }
      } else {
        this.components[componentIdStr] = new (_componentMap || _load_componentMap()).default({ files, origin });
        this.components[componentIdStr].mainFile = this._getMainFile((0, (_utils || _load_utils()).pathNormalizeToLinux)(mainFile), this.components[componentIdStr]);
      }
      if (rootDir) {
        var root = this._makePathRelativeToProjectRoot(rootDir);
        this.components[componentIdStr].rootDir = root ? (0, (_utils || _load_utils()).pathNormalizeToLinux)(root) : root;
      }
      if (originallySharedDir) {
        this.components[componentIdStr].originallySharedDir = originallySharedDir;
      }
      if (origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED || origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) {
        // if there are older versions, the user is updating an existing component, delete old ones from bit.map
        this.deleteOlderVersionsOfComponent(componentId);
      }

      return this.components[componentIdStr];
    }
  }, {
    key: '_removeFromComponentsArray',
    value: function _removeFromComponentsArray(componentId) {
      var _this4 = this;

      var invalidateCache = function invalidateCache() {
        _this4.paths = {};
      };
      delete this.components[componentId];
      invalidateCache();
    }
  }, {
    key: 'removeComponent',
    value: function removeComponent(id) {
      var bitId = id instanceof (_bitId || _load_bitId()).BitId ? id : (_bitId || _load_bitId()).BitId.parse(id);
      var bitmapComponent = this.getExistingComponentId(bitId.toStringWithoutScopeAndVersion());
      this._removeFromComponentsArray(bitmapComponent);
      return bitmapComponent;
    }
  }, {
    key: 'removeComponents',
    value: function removeComponents(ids) {
      var _this5 = this;

      return ids.map(function (id) {
        return _this5.removeComponent(id);
      });
    }
  }, {
    key: 'addMainDistFileToComponent',
    value: function addMainDistFileToComponent(id, distFilesPaths) {
      var _this6 = this;

      if (!this.components[id]) {
        (_logger || _load_logger()).default.warn(`unable to find the component ${id} in bit.map file`);
        return;
      }
      var distFilesPathsNormalized = distFilesPaths.map(function (filePath) {
        return (0, (_utils || _load_utils()).pathNormalizeToLinux)(filePath);
      });

      var mainDistFile = distFilesPathsNormalized.find(function (distFile) {
        return distFile.endsWith(_this6.components[id].mainFile);
      });
      if (!mainDistFile) {
        (_logger || _load_logger()).default.warn(`unable to find the main dist file of component ${id}. Dist files: ${distFilesPathsNormalized.join(', ')}`);
        return;
      }
      this.components[id].mainDistFile = this._makePathRelativeToProjectRoot(mainDistFile);
    }
  }, {
    key: 'isExistWithSameVersion',
    value: function isExistWithSameVersion(id) {
      return id.hasVersion() && this.components[id.toString()];
    }

    /**
     * needed after exporting a component.
     * We don't support export of nested components, only authored or imported. For authored/imported components, could be
     * in the file-system only one instance with the same box-name and component-name. As a result, we can strip the
     * scope-name and the version, find the older version in bit.map and update the id with the new one.
     */

  }, {
    key: 'updateComponentId',
    value: function updateComponentId(id) {
      var _this7 = this;

      var newIdString = id.toString();
      var olderComponentsIds = Object.keys(this.components).filter(function (componentId) {
        return (_bitId || _load_bitId()).BitId.parse(componentId).toStringWithoutScopeAndVersion() === id.toStringWithoutScopeAndVersion() && componentId !== newIdString && _this7.components[componentId].origin !== (_constants || _load_constants()).COMPONENT_ORIGINS.NESTED;
      });

      if (!olderComponentsIds.length) {
        (_logger || _load_logger()).default.debug(`bit-map: no need to update ${newIdString}`);
        return;
      }
      if (olderComponentsIds.length > 1) {
        throw new Error(`Your ${(_constants || _load_constants()).BIT_MAP} file has more than one version of ${id.toStringWithoutScopeAndVersion()} and they 
      are authored or imported. This scenario is not supported`);
      }
      var olderComponentId = olderComponentsIds[0];
      (_logger || _load_logger()).default.debug(`BitMap: updating an older component ${olderComponentId} with a newer component ${newIdString}`);
      this.components[newIdString] = this.components[olderComponentId];
      this._removeFromComponentsArray(olderComponentId);
    }
  }, {
    key: 'getComponent',
    value: function getComponent(id) {
      var shouldThrow = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if ((_ramda || _load_ramda()).default.is(String, id)) {
        id = (_bitId || _load_bitId()).BitId.parse(id);
      }
      // $FlowFixMe
      if (id.hasVersion()) {
        if (!this.components[id] && shouldThrow) throw new (_exceptions || _load_exceptions()).MissingBitMapComponent(id);
        return this.components[id];
      }
      var idWithVersion = Object.keys(this.components).find(
      // $FlowFixMe
      function (componentId) {
        return (_bitId || _load_bitId()).BitId.parse(componentId).toStringWithoutVersion() === id.toStringWithoutVersion();
      });
      if (!idWithVersion && shouldThrow) throw new (_exceptions || _load_exceptions()).MissingBitMapComponent(id);
      // $FlowFixMe
      return this.components[idWithVersion];
    }

    /**
     * Return a potential componentMap if file is supposed to be part of it
     * by a path exist in the files object
     *
     * @param {string} componentPath relative to consumer - as stored in bit.map files object
     * @returns {ComponentMap} componentMap
     */

  }, {
    key: 'getComponentObjectOfFileByPath',
    value: function getComponentObjectOfFileByPath(componentPath) {
      var _this8 = this;

      var components = this.getAllComponents();
      return (_ramda || _load_ramda()).default.pickBy(function (component) {
        return (0, (_utils || _load_utils()).pathIsInside)(componentPath, component.rootDir || _this8.projectRoot);
      }, components);
    }

    /**
     * Return a component id as listed in bit.map file
     * by a path exist in the files object
     *
     * @param {string} componentPath relative to consumer - as stored in bit.map files object
     * @returns {string} component id
     * @memberof BitMap
     */

  }, {
    key: 'getComponentIdByPath',
    value: function getComponentIdByPath(componentPath) {
      this._populateAllPaths();
      return this.paths[componentPath];
    }
  }, {
    key: '_populateAllPaths',
    value: function _populateAllPaths() {
      var _this9 = this;

      if ((_ramda || _load_ramda()).default.isEmpty(this.paths)) {
        Object.keys(this.components).forEach(function (componentId) {
          var component = _this9.components[componentId];
          component.files.forEach(function (file) {
            var relativeToConsumer = component.rootDir ? (0, (_utils || _load_utils()).pathJoinLinux)(component.rootDir, file.relativePath) : file.relativePath;
            _this9.paths[relativeToConsumer] = componentId;
          });
        });
      }
    }
  }, {
    key: 'modifyComponentsToLinuxPath',
    value: function modifyComponentsToLinuxPath(components) {
      Object.keys(components).forEach(function (key) {
        components[key].files.forEach(function (file) {
          file.relativePath = (0, (_utils || _load_utils()).pathNormalizeToLinux)(file.relativePath);
        });
        components[key].mainFile = (0, (_utils || _load_utils()).pathNormalizeToLinux)(components[key].mainFile);
      });
    }
  }, {
    key: 'updatePathLocation',
    value: function updatePathLocation(from, to, fromExists) {
      var _this10 = this;

      var existingPath = fromExists ? from : to;
      var isPathDir = (0, (_utils || _load_utils()).isDir)(existingPath);
      var allChanges = [];
      Object.keys(this.components).forEach(function (componentId) {
        var componentMap = _this10.components[componentId];
        var changes = isPathDir ? componentMap.updateDirLocation(from, to) : componentMap.updateFileLocation(from, to);
        if (changes && changes.length) allChanges.push({ id: componentId, changes });
      });
      if ((_ramda || _load_ramda()).default.isEmpty(allChanges)) {
        var errorMsg = isPathDir ? `neither one of the files use the directory ${from}` : `the file ${existingPath} is untracked`;
        throw new Error(errorMsg);
      }

      return allChanges;
    }
  }, {
    key: 'write',
    value: function write() {
      (_logger || _load_logger()).default.debug('writing to bit.map');
      this.modifyComponentsToLinuxPath(this.components);
      var bitMapContent = Object.assign({}, this.components, { version: this.version });
      return (0, (_utils || _load_utils()).outputFile)({ filePath: this.mapPath, content: JSON.stringify(bitMapContent, null, 4) });
    }
  }], [{
    key: 'ensure',
    value: function ensure(dirPath) {
      return Promise.resolve(this.load(dirPath));
    }
  }, {
    key: 'load',
    value: function load(dirPath) {
      // support old bitmaps
      var mapPath = (_fsExtra || _load_fsExtra()).default.existsSync((_path || _load_path()).default.join(dirPath, (_constants || _load_constants()).OLD_BIT_MAP)) && !(_fsExtra || _load_fsExtra()).default.existsSync((_path || _load_path()).default.join(dirPath, (_constants || _load_constants()).BIT_MAP)) ? (_path || _load_path()).default.join(dirPath, (_constants || _load_constants()).OLD_BIT_MAP) : (_path || _load_path()).default.join(dirPath, (_constants || _load_constants()).BIT_MAP);
      var components = {};
      var version = void 0;
      if ((_fsExtra || _load_fsExtra()).default.existsSync(mapPath)) {
        try {
          var mapFileContent = (_fsExtra || _load_fsExtra()).default.readFileSync(mapPath);
          var componentsJson = (_commentJson || _load_commentJson()).default.parse(mapFileContent.toString('utf8'), null, true);
          version = componentsJson.version;
          // Don't treat version like component
          delete componentsJson.version;
          Object.keys(componentsJson).forEach(function (componentId) {
            components[componentId] = (_componentMap || _load_componentMap()).default.fromJson(componentsJson[componentId]);
          });

          return new BitMap(dirPath, mapPath, components, version);
        } catch (e) {
          throw new (_exceptions || _load_exceptions()).InvalidBitMap(mapPath);
        }
      }
      (_logger || _load_logger()).default.info(`bit.map: unable to find an existing ${(_constants || _load_constants()).BIT_MAP} file. Will probably create a new one if needed`);
      return new BitMap(dirPath, mapPath, components, version || (_constants || _load_constants()).BIT_VERSION);
    }
  }]);

  return BitMap;
}();

exports.default = BitMap;