'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../logger/logger'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ComponentMap = function () {
  function ComponentMap(_ref) {
    var files = _ref.files,
        mainFile = _ref.mainFile,
        rootDir = _ref.rootDir,
        origin = _ref.origin,
        dependencies = _ref.dependencies,
        mainDistFile = _ref.mainDistFile,
        originallySharedDir = _ref.originallySharedDir;

    _classCallCheck(this, ComponentMap);

    this.files = files;
    this.mainFile = mainFile;
    this.rootDir = rootDir;
    this.origin = origin;
    this.dependencies = dependencies;
    this.mainDistFile = mainDistFile;
    this.originallySharedDir = originallySharedDir;
  }

  _createClass(ComponentMap, [{
    key: '_findFile',
    value: function _findFile(fileName) {
      var _this = this;

      return this.files.find(function (file) {
        var filePath = _this.rootDir ? (0, (_utils || _load_utils()).pathJoinLinux)(_this.rootDir, file.relativePath) : file.relativePath;
        return filePath === fileName;
      });
    }
  }, {
    key: 'updateFileLocation',
    value: function updateFileLocation(fileFrom, fileTo) {
      fileFrom = (0, (_utils || _load_utils()).pathNormalizeToLinux)(fileFrom);
      fileTo = (0, (_utils || _load_utils()).pathNormalizeToLinux)(fileTo);
      var currentFile = this._findFile(fileFrom);
      var changes = [];
      if (currentFile) {
        var _rootDir = this.rootDir;
        var newLocation = _rootDir ? ComponentMap.getPathWithoutRootDir(_rootDir, fileTo) : fileTo;
        (_logger || _load_logger()).default.debug(`updating file location from ${currentFile.relativePath} to ${newLocation}`);
        if (this.mainFile === currentFile.relativePath) this.mainFile = newLocation;
        changes.push({ from: currentFile.relativePath, to: newLocation });
        currentFile.relativePath = newLocation;
      }
      return changes;
    }
  }, {
    key: 'updateDirLocation',
    value: function updateDirLocation(dirFrom, dirTo) {
      var _this2 = this;

      dirFrom = (0, (_utils || _load_utils()).pathNormalizeToLinux)(dirFrom);
      dirTo = (0, (_utils || _load_utils()).pathNormalizeToLinux)(dirTo);
      var changes = [];
      if (this.rootDir && this.rootDir.startsWith(dirFrom)) {
        var newRootDir = this.rootDir.replace(dirFrom, dirTo);
        var newRootDirNormalized = (0, (_utils || _load_utils()).pathNormalizeToLinux)(newRootDir);
        changes.push({ from: this.rootDir, to: newRootDirNormalized });
        (_logger || _load_logger()).default.debug(`updating rootDir location from ${this.rootDir} to ${newRootDirNormalized}`);
        this.rootDir = newRootDirNormalized;
        return changes;
      }
      this.files.forEach(function (file) {
        var filePath = _this2.rootDir ? (_path || _load_path()).default.join(_this2.rootDir, file.relativePath) : file.relativePath;
        if (filePath.startsWith(dirFrom)) {
          var fileTo = filePath.replace(dirFrom, dirTo);
          var newLocation = _this2.rootDir ? ComponentMap.getPathWithoutRootDir(_this2.rootDir, fileTo) : fileTo;
          (_logger || _load_logger()).default.debug(`updating file location from ${file.relativePath} to ${newLocation}`);
          if (_this2.mainFile === file.relativePath) _this2.mainFile = newLocation;
          changes.push({ from: file.relativePath, to: newLocation });
          file.relativePath = newLocation;
        }
      });
      return changes;
    }
  }, {
    key: 'getFilesRelativeToConsumer',
    value: function getFilesRelativeToConsumer() {
      var _this3 = this;

      return this.files.map(function (file) {
        return _this3.rootDir ? (0, (_utils || _load_utils()).pathJoinLinux)(_this3.rootDir, file.relativePath) : file.relativePath;
      });
    }
  }, {
    key: 'getFilesGroupedByBeingTests',
    value: function getFilesGroupedByBeingTests() {
      var allFiles = [];
      var nonTestsFiles = [];
      var testsFiles = [];
      this.files.forEach(function (file) {
        allFiles.push(file.relativePath);
        if (file.test) testsFiles.push(file.relativePath);else nonTestsFiles.push(file.relativePath);
      });
      return { allFiles, nonTestsFiles, testsFiles };
    }
  }], [{
    key: 'fromJson',
    value: function fromJson(componentMapObj) {
      return new ComponentMap(componentMapObj);
    }
  }, {
    key: 'getPathWithoutRootDir',
    value: function getPathWithoutRootDir(rootDir, filePath) {
      var newPath = (0, (_utils || _load_utils()).pathRelativeLinux)(rootDir, filePath);
      if (newPath.startsWith('..')) {
        // this is forbidden for security reasons. Allowing files to be written outside the components directory may
        // result in overriding OS files.
        throw new Error(`unable to add file ${filePath} because it's located outside the component root dir ${rootDir}`);
      }
      return newPath;
    }
  }, {
    key: 'changeFilesPathAccordingToItsRootDir',
    value: function changeFilesPathAccordingToItsRootDir(existingRootDir, files) {
      var _this4 = this;

      var changes = [];
      files.forEach(function (file) {
        var newPath = _this4.getPathWithoutRootDir(existingRootDir, file.relativePath);
        changes.push({ from: file.relativePath, to: newPath });
        file.relativePath = newPath;
      });
      return changes;
    }
  }]);

  return ComponentMap;
}();

exports.default = ComponentMap;