'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _;

function _load_() {
  return _ = require('./');
}

var _dependency;

function _load_dependency() {
  return _dependency = require('./dependency');
}

var _constants;

function _load_constants() {
  return _constants = require('../../../constants');
}

var _consumer;

function _load_consumer() {
  return _consumer = _interopRequireDefault(require('../../consumer'));
}

var _consumerComponent;

function _load_consumerComponent() {
  return _consumerComponent = _interopRequireDefault(require('../consumer-component'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../bit-id');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../../../scope/scope'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Dependencies = function () {
  function Dependencies() {
    var dependencies = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    _classCallCheck(this, Dependencies);

    this.dependencies = this.deserialize(dependencies);
  }

  _createClass(Dependencies, [{
    key: 'serialize',
    value: function serialize() {
      return this.dependencies.map(function (dep) {
        return Object.assign({}, dep, { id: dep.id.toString() });
      });
    }
  }, {
    key: 'get',
    value: function get() {
      return this.dependencies;
    }
  }, {
    key: 'deserialize',
    value: function deserialize(dependencies) {
      return dependencies.map(function (dependency) {
        return {
          id: (_ramda || _load_ramda()).default.is(String, dependency.id) ? (_bitId || _load_bitId()).BitId.parse(dependency.id) : dependency.id,
          relativePaths: dependency.relativePaths || [
          // backward compatibility. (previously, it was "relativePath" without the ending 's' and was not an array.
          { sourceRelativePath: dependency.relativePath, destinationRelativePath: dependency.relativePath }]
        };
      });
    }
  }, {
    key: 'toStringOfIds',
    value: function toStringOfIds() {
      return this.dependencies.map(function (dep) {
        return dep.id.toString();
      });
    }
  }, {
    key: 'isEmpty',
    value: function isEmpty() {
      return !this.dependencies.length;
    }
  }, {
    key: 'asWritableObject',
    value: function asWritableObject() {
      return (_ramda || _load_ramda()).default.mergeAll(this.dependencies.map(function (dependency) {
        return dependency.id.toObject();
      }));
    }
  }, {
    key: 'cloneAsString',
    value: function cloneAsString() {
      return this.dependencies.map(function (dependency) {
        var dependencyClone = (_ramda || _load_ramda()).default.clone(dependency);
        dependencyClone.id = dependency.id.toString();
        return dependencyClone;
      });
    }
  }, {
    key: 'stripOriginallySharedDir',
    value: function stripOriginallySharedDir(bitMap, originallySharedDir) {
      var pathWithoutSharedDir = function pathWithoutSharedDir(pathStr, sharedDir) {
        if (!sharedDir) return pathStr;
        var partToRemove = `${sharedDir}/`;
        return pathStr.replace(partToRemove, '');
      };
      this.dependencies.forEach(function (dependency) {
        var dependencyId = dependency.id.toString();
        var depFromBitMap = bitMap.getComponent(dependencyId);
        dependency.relativePaths.forEach(function (relativePath) {
          relativePath.sourceRelativePath = pathWithoutSharedDir(relativePath.sourceRelativePath, originallySharedDir);
          if (depFromBitMap && depFromBitMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED) {
            relativePath.destinationRelativePath = pathWithoutSharedDir(relativePath.destinationRelativePath, depFromBitMap.originallySharedDir);
          }
        });
      });
    }
  }, {
    key: 'getSourcesPaths',
    value: function getSourcesPaths() {
      return (_ramda || _load_ramda()).default.flatten(this.dependencies.map(function (dependency) {
        return dependency.relativePaths.map(function (relativePath) {
          return relativePath.sourceRelativePath;
        });
      }));
    }
  }, {
    key: 'getDependenciesComponents',
    value: function getDependenciesComponents(consumer, component) {
      var _this = this;

      var getDependencies = function getDependencies() {
        return _this.dependencies.map(function (dependency) {
          if (consumer.bitMap.isExistWithSameVersion(dependency.id)) {
            return consumer.loadComponent(dependency.id);
          }
          // when dependencies are imported as npm packages, they are not in bit.map
          component.dependenciesSavedAsComponents = false;
          return consumer.scope.loadComponent(dependency.id, false);
        });
      };
      return Promise.all(getDependencies());
    }
  }, {
    key: 'addRemoteAndLocalVersions',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(scope, modelDependencies) {
        var dependenciesIds, localDependencies, remoteVersionsDependencies;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                dependenciesIds = this.dependencies.map(function (dependency) {
                  return dependency.id;
                });
                _context.next = 3;
                return scope.latestVersions(dependenciesIds);

              case 3:
                localDependencies = _context.sent;
                _context.next = 6;
                return scope.fetchRemoteVersions(dependenciesIds);

              case 6:
                remoteVersionsDependencies = _context.sent;


                this.dependencies.forEach(function (dependency) {
                  var dependencyIdWithoutVersion = dependency.id.toStringWithoutVersion();
                  var remoteVersionId = remoteVersionsDependencies.find(function (remoteId) {
                    return remoteId.toStringWithoutVersion() === dependencyIdWithoutVersion;
                  });
                  var localVersionId = localDependencies.find(function (localId) {
                    return localId.toStringWithoutVersion() === dependencyIdWithoutVersion;
                  });
                  var modelVersionId = modelDependencies.get().find(function (modelDependency) {
                    return modelDependency.id.toStringWithoutVersion() === dependencyIdWithoutVersion;
                  });
                  dependency.remoteVersion = remoteVersionId ? remoteVersionId.version : null;
                  dependency.localVersion = localVersionId ? localVersionId.version : null;
                  dependency.currentVersion = modelVersionId ? modelVersionId.id.version : dependency.id.version;
                });

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function addRemoteAndLocalVersions(_x2, _x3) {
        return _ref.apply(this, arguments);
      }

      return addRemoteAndLocalVersions;
    }()
  }]);

  return Dependencies;
}();

exports.default = Dependencies;
//# sourceMappingURL=dependencies.js.map