'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

var _logger2;

function _load_logger2() {
  return _logger2 = require('../logger/logger');
}

var _extensionCommand;

function _load_extensionCommand() {
  return _extensionCommand = _interopRequireDefault(require('./extension-command'));
}

var _environment;

function _load_environment() {
  return _environment = _interopRequireDefault(require('../environment'));
}

var _scope;

function _load_scope() {
  return _scope = require('../scope');
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../consumer');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _loader;

function _load_loader() {
  return _loader = _interopRequireDefault(require('../cli/loader'));
}

var _hooks;

function _load_hooks() {
  return _hooks = _interopRequireDefault(require('../hooks'));
}

var _hooks2;

function _load_hooks2() {
  return _hooks2 = require('../hooks');
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var HooksManagerInstance = (_hooks || _load_hooks()).default.getInstance();
var CORE_EXTENSIONS_PATH = './core-extensions';

/**
 * A class which represent an extension
 * The different attributes,
 * Extension API,
 * Load extension
 * Config
 */
var Extension = function () {
  function Extension(extensionProps) {
    var _this = this;

    _classCallCheck(this, Extension);

    this.api = {
      /**
       * API to resiter new command to bit
       */
      registerCommand: function registerCommand(newCommand) {
        // TODO: validate new command format
        (_logger || _load_logger()).default.info(`registering new command ${newCommand.name}`);
        _this.commands.push(new (_extensionCommand || _load_extensionCommand()).default(newCommand));
      },
      /**
       * API to register action to an existing hook (hook name might be a hook defined by another extension)
       */
      registerActionToHook: function registerActionToHook(hookName, hookAction) {
        (_logger || _load_logger()).default.info(`registering ${hookAction.name} to hook ${hookName}`);
        _this.registeredHooksActions[hookName] = hookAction;
      },
      /**
       * API to register a new hook name, usuful for communicate between different extensions
       */
      registerNewHook: function registerNewHook(hookName) {
        (_logger || _load_logger()).default.info(`registering new global hook ${hookName}`);
        _this.newHooks.push(hookName);
        // Register the new hook in the global hooks manager
        HooksManagerInstance.registerNewHook(hookName, { extension: _this.name });
      },
      /**
       * API to trigger a hook registered by this extension.
       * trigger hook are available only for hooks registered by you.
       */
      triggerHook: function triggerHook(hookName, args) {
        if (!(_ramda || _load_ramda()).default.contains(hookName, _this.newHooks)) {
          (_logger || _load_logger()).default.debug(`trying to trigger the hook ${hookName} which not registerd by this extension`);
          return;
        }
        HooksManagerInstance.triggerHook(hookName, args);
      },
      /**
       * API to get logger
       */
      getLogger: function getLogger() {
        return (0, (_logger2 || _load_logger2()).createExtensionLogger)(_this.name);
      },
      getLoader: function getLoader() {
        return (_loader || _load_loader()).default;
      },
      HOOKS_NAMES: _getHooksNames(),
      createIsolatedEnv: _createIsolatedEnv
    };

    this.name = extensionProps.name;
    this.rawConfig = extensionProps.rawConfig;
    this.options = extensionProps.options;
    this.dynamicConfig = extensionProps.rawConfig;
    this.commands = [];
    this.registeredHooksActions = {};
    this.newHooks = [];
  }

  /**
   * Load extension by name
   * The extension will be from scope by default or from file
   * if there is file(path) in the options
   * The file path is relative to the bit.json of the project or absolute
   * @param {string} name - name of the extension
   * @param {Object} rawConfig - raw config for the extension
   * @param {Object} options - extension options such as - disabled, file, core
   * @param {string} consumerPath - path to the consumer folder (to load the file relatively)
   * @param {string} scopePath - scope which stores the extension code
   */
  // Store the required plugin


  _createClass(Extension, [{
    key: 'registerHookActionsOnHooksManager',


    /**
     * Register the hooks on the global hooks manager
     * We don't do this directly on the api in order to be able to register to hooks defined by another extensions
     * So we want to make sure to first load and register all new hooks from all extensions and only then register the actions
     */
    value: function registerHookActionsOnHooksManager() {
      var _this2 = this;

      var registerAction = function registerAction(hookAction, hookName) {
        HooksManagerInstance.registerActionToHook(hookName, hookAction, { extension: _this2.name });
      };
      (_ramda || _load_ramda()).default.forEachObjIndexed(registerAction, this.registeredHooksActions);
    }
  }], [{
    key: 'load',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(name) {
        var rawConfig = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var consumerPath = arguments[3];
        var scopePath = arguments[4];
        var absPath, componentPath;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!options.file) {
                  _context.next = 4;
                  break;
                }

                absPath = options.file;

                if (!(_path || _load_path()).default.isAbsolute(options.file)) {
                  absPath = (_path || _load_path()).default.resolve(consumerPath, options.file);
                }
                return _context.abrupt('return', Extension.loadFromFile(name, absPath, rawConfig, options));

              case 4:
                _context.prev = 4;
                componentPath = _getExtensionPath(name, scopePath, options.core);
                return _context.abrupt('return', Extension.loadFromFile(name, componentPath, rawConfig, options));

              case 9:
                _context.prev = 9;
                _context.t0 = _context['catch'](4);

                (_logger || _load_logger()).default.error(`loading extension ${name} faild`);
                (_logger || _load_logger()).default.error(_context.t0);
                return _context.abrupt('return', null);

              case 14:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[4, 9]]);
      }));

      function load(_x3) {
        return _ref.apply(this, arguments);
      }

      return load;
    }()
  }, {
    key: 'loadFromFile',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(name, filePath) {
        var rawConfig = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
        var extension, script, msg;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                (_logger || _load_logger()).default.info(`loading extension ${name} from ${filePath}`);
                extension = new Extension({ name, rawConfig, options });
                // Skip disabled extensions

                if (!options.disabled) {
                  _context2.next = 7;
                  break;
                }

                extension.disabled = true;
                (_logger || _load_logger()).default.info(`skip extension ${name} because it is disabled`);
                extension.loaded = false;
                return _context2.abrupt('return', extension);

              case 7:
                extension.filePath = filePath;
                _context2.prev = 8;
                script = require(filePath);

                extension.script = script.default ? script.default : script;

                if (!(extension.script.getDynamicConfig && typeof extension.script.getDynamicConfig === 'function')) {
                  _context2.next = 15;
                  break;
                }

                _context2.next = 14;
                return extension.script.getDynamicConfig(rawConfig);

              case 14:
                extension.dynamicConfig = _context2.sent;

              case 15:
                if (!(extension.script.init && typeof extension.script.init === 'function')) {
                  _context2.next = 18;
                  break;
                }

                _context2.next = 18;
                return extension.script.init(rawConfig, extension.dynamicConfig, extension.api);

              case 18:
                _context2.next = 27;
                break;

              case 20:
                _context2.prev = 20;
                _context2.t0 = _context2['catch'](8);

                if (_context2.t0.code === 'MODULE_NOT_FOUND') {
                  msg = `loading extension ${name} faild, the file ${filePath} not found`;

                  (_logger || _load_logger()).default.error(msg);
                  console.error(msg);
                }
                (_logger || _load_logger()).default.error(`loading extension ${name} faild`);
                (_logger || _load_logger()).default.error(_context2.t0);
                extension.loaded = false;
                return _context2.abrupt('return', extension);

              case 27:
                extension.loaded = true;
                return _context2.abrupt('return', extension);

              case 29:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[8, 20]]);
      }));

      function loadFromFile(_x6, _x7) {
        return _ref2.apply(this, arguments);
      }

      return loadFromFile;
    }()
  }]);

  return Extension;
}();

exports.default = Extension;


var _createIsolatedEnv = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(scopePath, dirPath) {
    var scope, isolatedEnvironment;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return _loadScope(scopePath);

          case 2:
            scope = _context3.sent;
            isolatedEnvironment = new (_environment || _load_environment()).default(scope, dirPath);
            _context3.next = 6;
            return isolatedEnvironment.create();

          case 6:
            return _context3.abrupt('return', isolatedEnvironment);

          case 7:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function _createIsolatedEnv(_x8, _x9) {
    return _ref3.apply(this, arguments);
  };
}();

var _loadScope = function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(scopePath) {
    var consumer;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            if (!scopePath) {
              _context4.next = 2;
              break;
            }

            return _context4.abrupt('return', (0, (_scope || _load_scope()).loadScope)(scopePath));

          case 2:
            _context4.next = 4;
            return (0, (_consumer || _load_consumer()).loadConsumer)();

          case 4:
            consumer = _context4.sent;
            return _context4.abrupt('return', consumer.scope);

          case 6:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function _loadScope(_x10) {
    return _ref4.apply(this, arguments);
  };
}();

var _getCoreExtensionPath = function _getCoreExtensionPath(name) {
  var componentPath = (_path || _load_path()).default.join(__dirname, CORE_EXTENSIONS_PATH, name);
  return componentPath;
};

var _getRegularExtensionPath = function _getRegularExtensionPath(name, scopePath) {
  var bitId = (_bitId || _load_bitId()).BitId.parse(name);
  var internalComponentsPath = (_scope || _load_scope()).Scope.getComponentsRelativePath();
  var internalComponentPath = (_scope || _load_scope()).Scope.getComponentRelativePath(bitId);
  var componentPath = (_path || _load_path()).default.join(scopePath, internalComponentsPath, internalComponentPath);
  return componentPath;
};

var _getExtensionPath = function _getExtensionPath(name, scopePath) {
  var isCore = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  if (isCore) {
    return _getCoreExtensionPath(name);
  }
  return _getRegularExtensionPath(name, scopePath);
};

var _getHooksNames = function _getHooksNames() {
  var hooks = {};
  (_constants || _load_constants()).HOOKS_NAMES.forEach(function (hook) {
    hooks[hook] = hook;
  });
  return hooks;
};