'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getIndexFileName = exports.getLinkContent = exports.writeDependencyLinks = exports.writeEntryPointsForComponent = undefined;

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

/**
 * The following scenario will help understanding why links are needed.
 * Component A has a dependency B. (for instance, in a.js there is a require statement to 'b.js').
 * While importing component A, it knows about the B dependency and it saves it under 'dependencies' directory of A.
 * The problem is that the above require is broken, because 'b.js' is not in the same place where it was originally.
 * This function solves this issue by creating the 'b.js' file in the original location and points to the new location
 * under 'dependencies' of A.
 */
var writeDependencyLinks = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(componentDependencies, consumer, createNpmLinkFiles) {
    var consumerPath, prepareLinkFile, componentLink, componentLinks, allLinksP;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            consumerPath = consumer.getPath();

            prepareLinkFile = function prepareLinkFile(componentId, mainFile, linkPath, relativePathInDependency, relativePath, depRootDir, isNpmLink) {
              // this is used to convert the component name to a valid npm package  name
              var packagePath = `${(0, (_globalConfig || _load_globalConfig()).getSync)((_constants || _load_constants()).CFG_REGISTRY_DOMAIN_PREFIX) || (_constants || _load_constants()).DEFAULT_REGISTRY_DOMAIN_PREFIX}/${componentId.toStringWithoutVersion().replace(/\//g, '.')}`;
              var actualFilePath = depRootDir ? (_path || _load_path()).default.join(depRootDir, relativePathInDependency) : relativePathInDependency;
              if (relativePathInDependency === mainFile) {
                actualFilePath = depRootDir ? (_path || _load_path()).default.join(depRootDir, mainFile) : mainFile;
              }
              var relativeFilePath = (_path || _load_path()).default.relative((_path || _load_path()).default.dirname(linkPath), actualFilePath);
              var importSpecifiers = relativePath.importSpecifiers;
              var linkContent = getLinkContent(relativeFilePath, importSpecifiers, isNpmLink, packagePath);
              (_logger || _load_logger()).default.debug(`writeLinkFile, on ${linkPath}`);
              var linkPathExt = (0, (_utils || _load_utils()).getExt)(linkPath);
              var isEs6 = importSpecifiers && linkPathExt === 'js';
              return { linkPath, linkContent, isEs6 };
            };

            componentLink = function componentLink(depId, depComponent, relativePath, parentComponent, parentComponentMap) {
              var parentDir = parentComponent.writtenPath || parentComponentMap.rootDir; // when running from bit build, the writtenPath is not available
              var relativePathInDependency = (_path || _load_path()).default.normalize(relativePath.destinationRelativePath);
              var mainFile = depComponent.dists.calculateMainDistFile(depComponent.mainFile);
              var hasDist = parentComponent.dists.writeDistsFiles && !parentComponent.dists.isEmpty();
              var distRoot = parentComponent.dists.getDistDirForConsumer(consumer, parentComponentMap.rootDir);

              var relativeDistPathInDependency = (0, (_utils || _load_utils()).searchFilesIgnoreExt)(depComponent.dists.get(), relativePathInDependency, 'relative');
              relativeDistPathInDependency = relativeDistPathInDependency ? relativeDistPathInDependency.relative : relativePathInDependency;

              var relativeDistExtInDependency = (0, (_utils || _load_utils()).getExt)(relativeDistPathInDependency);
              var sourceRelativePath = relativePath.sourceRelativePath;
              var linkPath = (_path || _load_path()).default.join(parentDir, sourceRelativePath);
              var distLinkPath = void 0;
              var linkFiles = [];
              var depComponentMap = parentComponent.dependenciesSavedAsComponents ? consumer.bitMap.getComponent(depId, true) : undefined;

              var depRootDir = depComponentMap ? (_path || _load_path()).default.join(consumerPath, depComponentMap.rootDir) : undefined;
              var isNpmLink = createNpmLinkFiles || !parentComponent.dependenciesSavedAsComponents;
              if (hasDist) {
                var sourceRelativePathWithCompiledExt = `${(0, (_utils || _load_utils()).getWithoutExt)(sourceRelativePath)}.${relativeDistExtInDependency}`;
                var depRootDirDist = depComponentMap ? depComponent.dists.getDistDirForConsumer(consumer, depComponentMap.rootDir) : undefined;
                distLinkPath = (_path || _load_path()).default.join(distRoot, sourceRelativePathWithCompiledExt);
                // Generate a link file inside dist folder of the dependent component
                var _linkFile = prepareLinkFile(depId, mainFile, distLinkPath, relativeDistPathInDependency, relativePath, depRootDirDist, isNpmLink);
                linkFiles.push(_linkFile);
              }

              var linkFile = prepareLinkFile(depId, mainFile, linkPath, relativePathInDependency, relativePath, depRootDir, isNpmLink);
              linkFiles.push(linkFile);
              return linkFiles;
            };

            componentLinks = function componentLinks(dependencies, parentComponent, parentComponentMap) {
              var directDependencies = parentComponent.getAllDependencies();
              var flattenedDependencies = parentComponent.getAllFlattenedDependencies();
              if (!directDependencies || !directDependencies.length) return [];
              var links = directDependencies.map(function (dep) {
                if (!dep.relativePaths || (_ramda || _load_ramda()).default.isEmpty(dep.relativePaths)) return [];
                var resolveDepVersion = dep.id;
                // Check if the dependency is latest, if yes we need to resolve if from the flatten dependencies to get the
                // Actual version number, because on the bitmap we have only specific versions
                if (dep.id.getVersion().latest) {
                  resolveDepVersion = flattenedDependencies.resolveVersion(dep.id).toString();
                }

                // Helper function to look for the component object
                var _byComponentId = function _byComponentId(dependency) {
                  return dependency.id.toString() === resolveDepVersion.toString();
                };
                // Get the real dependency component
                var depComponent = (_ramda || _load_ramda()).default.find(_byComponentId, dependencies);

                var currLinks = dep.relativePaths.map(function (relativePath) {
                  return componentLink(resolveDepVersion, depComponent, relativePath, parentComponent, parentComponentMap);
                });
                return (_ramda || _load_ramda()).default.flatten(currLinks);
              });
              var flattenLinks = (_ramda || _load_ramda()).default.flatten(links);
              var groupLinks = (0, (_lodash || _load_lodash()).default)(flattenLinks, function (link) {
                return link.linkPath;
              });
              var allLinksP = Object.keys(groupLinks).map(function (group) {
                var content = '';
                if (groupLinks[group][0].isEs6) {
                  // check by the first item of the array, it can be any other item as well
                  content = 'Object.defineProperty(exports, "__esModule", { value: true });\n';
                }
                content += groupLinks[group].map(function (linkItem) {
                  return linkItem.linkContent;
                }).join('\n');
                return (0, (_utils || _load_utils()).outputFile)({ filePath: group, content });
              });

              return Promise.all(allLinksP);
            };

            allLinksP = componentDependencies.map(function (componentWithDeps) {
              var componentMap = consumer.bitMap.getComponent(componentWithDeps.component.id, true);
              if (componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) {
                (_logger || _load_logger()).default.debug(`writeDependencyLinks, ignoring a component ${componentWithDeps.component.id} as it is an author component`);
                return Promise.resolve();
              }
              // it must be IMPORTED. We don't pass NESTED to this function
              (_logger || _load_logger()).default.debug(`writeDependencyLinks, generating links for ${componentWithDeps.component.id}`);
              componentWithDeps.component.stripOriginallySharedDir(consumer.bitMap);

              var directLinksP = componentLinks(componentWithDeps.allDependencies, componentWithDeps.component, componentMap);

              if (componentWithDeps.component.dependenciesSavedAsComponents) {
                var indirectLinksP = componentWithDeps.allDependencies.map(function (dep) {
                  var depComponentMap = consumer.bitMap.getComponent(dep.id, true);
                  // We pass here the componentWithDeps.dependencies again because it contains the full dependencies objects
                  // also the indirect ones
                  // The dep.dependencies contain only an id and relativePaths and not the full object
                  return componentLinks(componentWithDeps.allDependencies, dep, depComponentMap);
                });
                return Promise.all([directLinksP].concat(_toConsumableArray(indirectLinksP)));
              }
              return directLinksP;
            });
            return _context.abrupt('return', Promise.all(allLinksP));

          case 6:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function writeDependencyLinks(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Relevant for IMPORTED and NESTED only
 */


var writeEntryPointsForComponent = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(component, consumer) {
    var componentId, componentMap, componentRoot, mainFile, indexName, entryPointFileContent, entryPointPath, distDir, entryPointDist;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            componentId = component.id.toString();
            componentMap = consumer.bitMap.getComponent(componentId);
            componentRoot = component.writtenPath || componentMap.rootDir;

            if (!(componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED)) {
              _context2.next = 5;
              break;
            }

            return _context2.abrupt('return', Promise.resolve());

          case 5:
            mainFile = component.dists.calculateMainDistFile(component.mainFile);
            indexName = getIndexFileName(mainFile); // Move to bit-javascript

            entryPointFileContent = getLinkContent(`./${mainFile}`);
            entryPointPath = (_path || _load_path()).default.join(componentRoot, indexName);

            if (!(!component.dists.isEmpty() && component.dists.writeDistsFiles && !consumer.shouldDistsBeInsideTheComponent())) {
              _context2.next = 15;
              break;
            }

            distDir = component.dists.getDistDirForConsumer(consumer, componentMap.rootDir);
            entryPointDist = (_path || _load_path()).default.join(distDir, indexName);

            (_logger || _load_logger()).default.debug(`writeEntryPointFile, on ${entryPointDist}`);
            _context2.next = 15;
            return (0, (_utils || _load_utils()).outputFile)({ filePath: entryPointDist, content: entryPointFileContent, override: false });

          case 15:
            (_logger || _load_logger()).default.debug(`writeEntryPointFile, on ${entryPointPath}`);
            return _context2.abrupt('return', (0, (_utils || _load_utils()).outputFile)({ filePath: entryPointPath, content: entryPointFileContent, override: false }));

          case 17:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function writeEntryPointsForComponent(_x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _normalizePath;

function _load_normalizePath() {
  return _normalizePath = _interopRequireDefault(require('normalize-path'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _lodash;

function _load_lodash() {
  return _lodash = _interopRequireDefault(require('lodash.groupby'));
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

var _scope;

function _load_scope() {
  return _scope = require('../scope');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../consumer/component'));
}

var _dependencies;

function _load_dependencies() {
  return _dependencies = require('../consumer/component/dependencies');
}

var _dependency;

function _load_dependency() {
  return _dependency = require('../consumer/component/dependencies/dependency');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _fileTypesPlugins;

function _load_fileTypesPlugins() {
  return _fileTypesPlugins = _interopRequireDefault(require('../plugins/file-types-plugins'));
}

var _globalConfig;

function _load_globalConfig() {
  return _globalConfig = require('../api/consumer/lib/global-config');
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../consumer');
}

var _componentMap;

function _load_componentMap() {
  return _componentMap = _interopRequireDefault(require('../consumer/bit-map/component-map'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var LINKS_CONTENT_TEMPLATES = {
  js: "module.exports = require('{filePath}');",
  ts: "export * from '{filePath}';",
  jsx: "export * from '{filePath}';",
  tsx: "export * from '{filePath}';",
  css: "@import '{filePath}.css';",
  scss: "@import '{filePath}.scss';",
  sass: "@import '{filePath}.sass';",
  less: "@import '{filePath}.less';"
};

var PACKAGES_LINKS_CONTENT_TEMPLATES = {
  css: "@import '~{filePath}';",
  scss: "@import '~{filePath}';",
  sass: "@import '~{filePath}';",
  less: "@import '~{filePath}';",
  'st.css': ':import { -st-from: "{filePath}";}'
};

var fileExtentionsForNpmLinkGenerator = ['js', 'ts', 'jsx', 'tsx'];

// todo: move to bit-javascript
function getIndexFileName(mainFile) {
  return `${(_constants || _load_constants()).DEFAULT_INDEX_NAME}.${(0, (_utils || _load_utils()).getExt)(mainFile)}`;
}

// todo: move to bit-javascript
function getLinkContent(filePath, importSpecifiers, createNpmLinkFiles, bitPackageName) {
  var fileExt = (0, (_utils || _load_utils()).getExt)(filePath);
  /**
   * Get the template for the generated link file.
   *
   * For ES6 and TypeScript the template is more complicated and we often need to know how originally the variables were
   * imported, whether default (e.g. import foo from './bar') or non-default (e.g. import { foo } from './bar').
   *
   * The importSpecifier.linkFile attribute exists when the main-file doesn't require the variable directly, but uses a
   * link-file to require it indirectly. E.g. src/bar.js: `import foo from './utils;` utils/index.js: `import foo from './foo';`
   */
  var getTemplate = function getTemplate() {
    if (importSpecifiers && importSpecifiers.length) {
      if (fileExt === 'js' || fileExt === 'jsx') {
        // @see e2e/flows/es6-link-files.e2e.js file for cases covered by the following snippet
        return importSpecifiers.map(function (importSpecifier) {
          if (!importSpecifier.linkFile) {
            // when no link-file is involved, use the standard non-es6 syntax (a privilege that doesn't exist for TS)
            return LINKS_CONTENT_TEMPLATES.js;
          }
          // for link files we need to know whether the main-file imports the variable as default or non-default
          var exportPart = 'exports';
          if (importSpecifier.mainFile.isDefault) {
            exportPart += '.default';
          } else {
            exportPart += `.${importSpecifier.mainFile.name}`;
          }
          var pathPart = "require('{filePath}')";

          if (importSpecifier.linkFile.isDefault) {
            pathPart += '.default';
          } else {
            pathPart += `.${importSpecifier.mainFile.name}`;
          }

          return `${exportPart} = ${pathPart};`;
        }).join('\n');
      } else if (fileExt === 'ts' || fileExt === 'tsx') {
        return importSpecifiers.map(function (importSpecifier) {
          var importPart = 'import ';
          if (importSpecifier.linkFile && importSpecifier.linkFile.isDefault || !importSpecifier.linkFile && importSpecifier.mainFile.isDefault) {
            importPart += `${importSpecifier.mainFile.name}`;
          } else {
            importPart += `{ ${importSpecifier.mainFile.name} }`;
          }
          importPart += " from '{filePath}';";

          var exportPart = 'export ';
          if (importSpecifier.mainFile.isDefault) {
            exportPart += `default ${importSpecifier.mainFile.name};`;
          } else {
            exportPart += `{ ${importSpecifier.mainFile.name} };`;
          }
          return `${importPart}\n${exportPart}`;
        }).join('\n');
      }
    }
    (_fileTypesPlugins || _load_fileTypesPlugins()).default.forEach(function (plugin) {
      LINKS_CONTENT_TEMPLATES[plugin.getExtension()] = plugin.getTemplate(importSpecifiers);
    });

    if (createNpmLinkFiles && !fileExtentionsForNpmLinkGenerator.includes(fileExt)) {
      return PACKAGES_LINKS_CONTENT_TEMPLATES[fileExt];
    }
    return LINKS_CONTENT_TEMPLATES[fileExt];
  };

  if (!filePath.startsWith('.')) {
    filePath = `./${filePath}`; // it must be relative, otherwise, it'll search it in node_modules
  }

  var filePathWithoutExt = (0, (_utils || _load_utils()).getWithoutExt)(filePath);
  var template = getTemplate();
  if (createNpmLinkFiles) {
    filePathWithoutExt = bitPackageName;
  } else {
    filePathWithoutExt = (0, (_utils || _load_utils()).getWithoutExt)(filePath); // remove the extension
  }

  if (!template) {
    throw new Error(`no template was found for ${filePath}, because .${fileExt} extension is not supported`);
  }
  return template.replace(/{filePath}/g, (0, (_normalizePath || _load_normalizePath()).default)(filePathWithoutExt));
}exports.writeEntryPointsForComponent = writeEntryPointsForComponent;
exports.writeDependencyLinks = writeDependencyLinks;
exports.getLinkContent = getLinkContent;
exports.getIndexFileName = getIndexFileName;
//# sourceMappingURL=link-generator.js.map