'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _execa;

function _load_execa() {
  return _execa = _interopRequireDefault(require('execa'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _ramda2;

function _load_ramda2() {
  return _ramda2 = require('ramda');
}

var _chalk;

function _load_chalk() {
  return _chalk = _interopRequireDefault(require('chalk'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../logger/logger'));
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var objectToArray = function objectToArray(obj) {
  return (0, (_ramda2 || _load_ramda2()).map)((0, (_ramda2 || _load_ramda2()).join)('@'), (0, (_ramda2 || _load_ramda2()).toPairs)(obj));
};
var rejectNils = (_ramda || _load_ramda()).default.reject((_ramda2 || _load_ramda2()).isNil);

var defaultNpmArgs = [];
var defaultYarnArgs = [];
var defaultPackageManagerArgs = {
  npm: defaultNpmArgs,
  yarn: defaultYarnArgs
};
var defaultPackageManagerProcessOptions = {
  cwd: process.cwd
};

var stripNonNpmErrors = function stripNonNpmErrors(errors) {
  // a workaround to remove all 'npm warn' and 'npm notice'.
  // NPM itself returns them even when --loglevel = error or when --silent/--quiet flags are set
  return errors.split('\n').filter(function (error) {
    return error.startsWith('npm ERR!') || error.startsWith('error');
  }).join('\n');
};

/**
 * Pick only allowed to be overriden options
 * @param {Object} userOptions
 */
var getAllowdPackageManagerProcessOptions = function getAllowdPackageManagerProcessOptions(userOptions) {
  var allowdOptions = ['shell', 'env', 'extendEnv', 'uid', 'gid', 'preferLocal', 'localDir', 'timeout'];
  return (_ramda || _load_ramda()).default.pick(allowdOptions, userOptions);
};

/**
 * Install packages in specific directory
 */
var _installInOneDirectory = function _installInOneDirectory(_ref) {
  var _ref$modules = _ref.modules,
      modules = _ref$modules === undefined ? [] : _ref$modules,
      _ref$packageManager = _ref.packageManager,
      packageManager = _ref$packageManager === undefined ? (_constants || _load_constants()).DEFAULT_PACKAGE_MANAGER : _ref$packageManager,
      _ref$packageManagerAr = _ref.packageManagerArgs,
      packageManagerArgs = _ref$packageManagerAr === undefined ? [] : _ref$packageManagerAr,
      _ref$packageManagerPr = _ref.packageManagerProcessOptions,
      packageManagerProcessOptions = _ref$packageManagerPr === undefined ? {} : _ref$packageManagerPr,
      dir = _ref.dir,
      _ref$verbose = _ref.verbose,
      verbose = _ref$verbose === undefined ? false : _ref$verbose;

  // Handle process options
  var allowedPackageManagerProcessOptions = getAllowdPackageManagerProcessOptions(packageManagerProcessOptions);
  var concretePackageManagerProcessOptions = (0, (_ramda2 || _load_ramda2()).merge)(defaultPackageManagerProcessOptions, allowedPackageManagerProcessOptions);
  concretePackageManagerProcessOptions.cwd = dir || concretePackageManagerProcessOptions.cwd;
  var cwd = concretePackageManagerProcessOptions.cwd;

  // taking care of object case
  var processedModules = (0, (_ramda2 || _load_ramda2()).is)(Object, modules) && !Array.isArray(modules) ? objectToArray(modules) : modules;

  // Handle process args
  var concretePackageManagerDefaultArgs = ['install'].concat(_toConsumableArray(processedModules), _toConsumableArray(defaultPackageManagerArgs[packageManager]));
  var concretePackageManagerArgs = rejectNils((_ramda || _load_ramda()).default.concat(concretePackageManagerDefaultArgs, packageManagerArgs));

  // Add npm verbose flag
  if (verbose && packageManager === 'npm') {
    // we may want to use it later. For now, it print too much information
    // concretePackageManagerArgs.push('--verbose');
  }

  (_fsExtra || _load_fsExtra()).default.ensureDirSync((_path || _load_path()).default.join(cwd, 'node_modules'));
  (_logger || _load_logger()).default.debug(`installing npm packages using ${packageManager} at ${cwd} with options:`, concretePackageManagerProcessOptions, `and args: ${concretePackageManagerArgs}`);

  // Set the shell to true to prevent problems with post install scripts when running as root
  var packageManagerClientName = packageManager;
  var childProcess = (0, (_execa || _load_execa()).default)(packageManagerClientName, concretePackageManagerArgs, concretePackageManagerProcessOptions);

  return childProcess.then(function (_ref2) {
    var stdout = _ref2.stdout,
        stderr = _ref2.stderr;

    // Remove the install from args since it's always there
    var printArgs = concretePackageManagerArgs.filter(function (arg) {
      return arg !== 'install';
    });
    var argsString = printArgs && printArgs.length > 0 ? `with args: ${printArgs}` : '';
    stdout = verbose ? stdout : `successfully ran ${packageManager} install at ${cwd} ${argsString}`;
    stderr = verbose ? stderr : '';
    return { stdout, stderr };
  }).catch(function (err) {
    var stderr = verbose ? err.stderr : stripNonNpmErrors(err.stderr);
    // return Promise.reject(`${stderr}\n\n${err.message}`);
    return Promise.reject(`${stderr}`);
  });
};

/**
 * when modules is empty, it runs 'npm install' without any package, which installs according to package.json file
 */
var installAction = function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(_ref3) {
    var modules = _ref3.modules,
        _ref3$packageManager = _ref3.packageManager,
        packageManager = _ref3$packageManager === undefined ? (_constants || _load_constants()).DEFAULT_PACKAGE_MANAGER : _ref3$packageManager,
        _ref3$packageManagerA = _ref3.packageManagerArgs,
        packageManagerArgs = _ref3$packageManagerA === undefined ? [] : _ref3$packageManagerA,
        _ref3$packageManagerP = _ref3.packageManagerProcessOptions,
        packageManagerProcessOptions = _ref3$packageManagerP === undefined ? {} : _ref3$packageManagerP,
        _ref3$useWorkspaces = _ref3.useWorkspaces,
        useWorkspaces = _ref3$useWorkspaces === undefined ? false : _ref3$useWorkspaces,
        _ref3$dirs = _ref3.dirs,
        dirs = _ref3$dirs === undefined ? [] : _ref3$dirs,
        rootDir = _ref3.rootDir,
        _ref3$installRootPack = _ref3.installRootPackageJson,
        installRootPackageJson = _ref3$installRootPack === undefined ? false : _ref3$installRootPack,
        _ref3$verbose = _ref3.verbose,
        verbose = _ref3$verbose === undefined ? false : _ref3$verbose;
    var results, rootDirResults, promises;
    return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!(useWorkspaces && packageManager === 'yarn')) {
              _context.next = 2;
              break;
            }

            return _context.abrupt('return', _installInOneDirectory({
              modules,
              packageManager,
              packageManagerArgs,
              packageManagerProcessOptions,
              dir: rootDir,
              verbose
            }));

          case 2:
            results = [];

            if (!installRootPackageJson) {
              _context.next = 8;
              break;
            }

            _context.next = 6;
            return _installInOneDirectory({
              modules,
              packageManager,
              packageManagerArgs,
              packageManagerProcessOptions,
              dir: rootDir,
              verbose
            });

          case 6:
            rootDirResults = _context.sent;

            results.push(rootDirResults);

          case 8:
            promises = dirs.map(function (dir) {
              return _installInOneDirectory({ modules, packageManager, packageManagerArgs, packageManagerProcessOptions, dir, verbose });
            });
            _context.t0 = results;
            _context.next = 12;
            return Promise.all(promises);

          case 12:
            _context.t1 = _context.sent;
            return _context.abrupt('return', _context.t0.concat.call(_context.t0, _context.t1));

          case 14:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function installAction(_x) {
    return _ref4.apply(this, arguments);
  };
}();

var printResults = function printResults(_ref5) {
  var stdout = _ref5.stdout,
      stderr = _ref5.stderr;

  console.log((_chalk || _load_chalk()).default.yellow(stdout)); // eslint-disable-line
  console.log((_chalk || _load_chalk()).default.yellow(stderr)); // eslint-disable-line
};

exports.default = {
  install: installAction,
  printResults
};
//# sourceMappingURL=npm-client.js.map