'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();
// import { BitId, BitIds } from '../../bit-id';


var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _execa;

function _load_execa() {
  return _execa = _interopRequireDefault(require('execa'));
}

var _graphviz;

function _load_graphviz() {
  return _graphviz = _interopRequireDefault(require('graphviz'));
}

var _objects;

function _load_objects() {
  return _objects = require('../objects');
}

var _models;

function _load_models() {
  return _models = require('../models');
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../logger/logger'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var defaultConfig = {
  layout: 'dot',
  fontName: 'Arial',
  fontSize: '14px',
  backgroundColor: '#000000',
  nodeColor: '#c6c5fe',
  noDependencyColor: '#cfffac',
  edgeColor: '#757575',
  graphVizOptions: null,
  graphVizPath: null
};

var VisualDependencyGraph = function () {
  function VisualDependencyGraph(repository, graph, config, rawOptions) {
    _classCallCheck(this, VisualDependencyGraph);

    this.repository = repository;
    this.graph = graph;
    this.config = config;
    this.rawOptions = rawOptions;
  }

  _createClass(VisualDependencyGraph, [{
    key: 'image',


    /**
     * Creates an image from the module dependency graph.
     * @param  {String} imagePath
     * @return {Promise}
     */
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(imagePath) {
        var _this = this;

        var options, outputP, image;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                // console.log('imagePath', imagePath)
                options = createGraphvizOptions(this.config);

                options.type = (_path || _load_path()).default.extname(imagePath).replace('.', '') || 'png';

                outputP = new Promise(function (resolve, reject) {
                  _this.graph.output(options, resolve, function (code, out, err) {
                    (_logger || _load_logger()).default.debug('Error during viz graph output function');
                    (_logger || _load_logger()).default.debug(code, out, err);
                    reject(new Error(err));
                  });
                });
                _context.next = 5;
                return outputP;

              case 5:
                image = _context.sent;
                _context.next = 8;
                return (_fsExtra || _load_fsExtra()).default.writeFile(imagePath, image);

              case 8:
                return _context.abrupt('return', (_path || _load_path()).default.resolve(imagePath));

              case 9:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function image(_x) {
        return _ref.apply(this, arguments);
      }

      return image;
    }()

    /**
     * Return the module dependency graph as DOT output.
     * @return {dot}
     */

  }, {
    key: 'dot',
    value: function dot() {
      return this.graph.to_dot();
    }
  }], [{
    key: 'load',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(repository) {
        var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var rawOptions = arguments[2];
        var concreteConfig, graph;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                concreteConfig = Object.assign({}, defaultConfig, config);

                checkGraphvizInstalled(config.graphVizPath);
                _context2.next = 4;
                return VisualDependencyGraph.buildDependenciesGraph(repository, concreteConfig);

              case 4:
                graph = _context2.sent;
                return _context2.abrupt('return', new VisualDependencyGraph(repository, graph, concreteConfig, rawOptions));

              case 6:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function load(_x3) {
        return _ref2.apply(this, arguments);
      }

      return load;
    }()

    /**
     * Creates the graphviz graph.
     * @param  {Object} modules
     * @param  {Array} circular
     * @param  {Object} config
     * @return {Promise}
     */

  }, {
    key: 'buildDependenciesGraph',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee5(repository, config) {
        var _this2 = this;

        var graph, nodes, depObj, allComponents;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                graph = (_graphviz || _load_graphviz()).default.digraph('G');
                nodes = {};


                if (config.graphVizPath) {
                  graph.setGraphVizPath(config.graphVizPath);
                }

                depObj = {};
                _context5.next = 6;
                return repository.listComponents(false);

              case 6:
                allComponents = _context5.sent;
                _context5.next = 9;
                return Promise.all(allComponents.map(function () {
                  var _ref4 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee4(component) {
                    return (_regenerator || _load_regenerator()).default.wrap(function _callee4$(_context4) {
                      while (1) {
                        switch (_context4.prev = _context4.next) {
                          case 0:
                            _context4.next = 2;
                            return Promise.all(Object.keys(component.versions).map(function () {
                              var _ref5 = _asyncToGenerator( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee3(version) {
                                var componentVersion, nodeId;
                                return (_regenerator || _load_regenerator()).default.wrap(function _callee3$(_context3) {
                                  while (1) {
                                    switch (_context3.prev = _context3.next) {
                                      case 0:
                                        _context3.next = 2;
                                        return component.loadVersion(version, repository);

                                      case 2:
                                        componentVersion = _context3.sent;

                                        if (componentVersion) {
                                          _context3.next = 5;
                                          break;
                                        }

                                        return _context3.abrupt('return');

                                      case 5:
                                        nodeId = `${component.id()}${(_constants || _load_constants()).VERSION_DELIMITER}${version}`;
                                        // nodes[nodeId] = nodes[nodeId] || graph.addNode(nodeId, componentVersion);

                                        nodes[nodeId] = nodes[nodeId] || graph.addNode(nodeId);
                                        if (componentVersion.dependencies.isEmpty()) {
                                          setNodeColor(nodes[nodeId], config.noDependencyColor);
                                        }
                                        // graph.setNode(`${component.id()}@${version}`, componentVersion);
                                        // graph.setParent(`${component.id()}@${version}`, component.id());
                                        componentVersion.id = component.toBitId();
                                        depObj[nodeId] = componentVersion;

                                      case 10:
                                      case 'end':
                                        return _context3.stop();
                                    }
                                  }
                                }, _callee3, _this2);
                              }));

                              return function (_x7) {
                                return _ref5.apply(this, arguments);
                              };
                            }()));

                          case 2:
                          case 'end':
                            return _context4.stop();
                        }
                      }
                    }, _callee4, _this2);
                  }));

                  return function (_x6) {
                    return _ref4.apply(this, arguments);
                  };
                }()));

              case 9:
                // set all edges
                // @todo: currently the label is "require". Change it to be "direct" and "indirect" depends on whether it comes from
                // flattenedDependencies or from dependencies.
                Object.keys(depObj).forEach(function (id) {
                  // TODO: add dev deps (with different edge color)
                  var deps = depObj[id].dependencies.toStringOfIds();
                  deps.forEach(function (dep) {
                    return graph.addEdge(id, dep);
                  });
                });
                return _context5.abrupt('return', Promise.resolve(graph));

              case 11:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function buildDependenciesGraph(_x4, _x5) {
        return _ref3.apply(this, arguments);
      }

      return buildDependenciesGraph;
    }()
  }]);

  return VisualDependencyGraph;
}();

/**
 * Set color on a node.
 * @param  {Object} node
 * @param  {String} color
 */


exports.default = VisualDependencyGraph;
function setNodeColor(node, color) {
  node.set('color', color);
  node.set('fontcolor', color);
}

/**
 * Check if Graphviz is installed on the system.
 * @param  {Object} config
 * @return {Promise}
 */
function checkGraphvizInstalled(graphVizPath) {
  var options = {
    shell: true
  };
  if (graphVizPath) {
    options.cwd = graphVizPath;
  }

  var childProcess = (0, (_execa || _load_execa()).default)('gvpr', ['-V'], options);
  return childProcess.catch(function (e) {
    (_logger || _load_logger()).default.debug(`Graphviz could not be found in path: ${graphVizPath || 'default path'}`);
    throw new Error(`Graphviz could not be found. Ensure that "gvpr" is in your $PATH.\n${e}`);
  });
}

/**
 * Return options to use with graphviz digraph.
 * @param  {Object} config
 * @return {Object}
 */
function createGraphvizOptions(config) {
  var graphVizOptions = config.graphVizOptions || {};

  return {
    G: Object.assign({
      overlap: false,
      pad: 0.111,
      layout: config.layout,
      bgcolor: config.backgroundColor
    }, graphVizOptions.G),
    E: Object.assign({
      color: config.edgeColor
    }, graphVizOptions.E),
    N: Object.assign({
      fontname: config.fontName,
      fontsize: config.fontSize,
      color: config.nodeColor,
      fontcolor: config.nodeColor
    }, graphVizOptions.N)
  };
}
//# sourceMappingURL=vizgraph.js.map