'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _objects;

function _load_objects() {
  return _objects = require('../objects');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope'));
}

var _source;

function _load_source() {
  return _source = _interopRequireDefault(require('./source'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../consumer/component'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _componentVersion;

function _load_componentVersion() {
  return _componentVersion = _interopRequireDefault(require('../component-version'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _dependencies;

function _load_dependencies() {
  return _dependencies = require('../../consumer/component/dependencies');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Version = function (_BitObject) {
  _inherits(Version, _BitObject);

  function Version(_ref) {
    var mainFile = _ref.mainFile,
        files = _ref.files,
        dists = _ref.dists,
        compiler = _ref.compiler,
        tester = _ref.tester,
        log = _ref.log,
        dependencies = _ref.dependencies,
        devDependencies = _ref.devDependencies,
        docs = _ref.docs,
        ci = _ref.ci,
        specsResults = _ref.specsResults,
        flattenedDependencies = _ref.flattenedDependencies,
        flattenedDevDependencies = _ref.flattenedDevDependencies,
        packageDependencies = _ref.packageDependencies,
        devPackageDependencies = _ref.devPackageDependencies,
        bindingPrefix = _ref.bindingPrefix;

    _classCallCheck(this, Version);

    var _this = _possibleConstructorReturn(this, (Version.__proto__ || Object.getPrototypeOf(Version)).call(this));

    _this.mainFile = mainFile;
    _this.files = files;
    _this.dists = dists;
    _this.compiler = compiler;
    _this.tester = tester;
    _this.log = log;
    _this.dependencies = new (_dependencies || _load_dependencies()).Dependencies(dependencies);
    _this.devDependencies = new (_dependencies || _load_dependencies()).Dependencies(devDependencies);
    _this.docs = docs;
    _this.ci = ci || {};
    _this.specsResults = specsResults;
    _this.flattenedDependencies = flattenedDependencies || new (_bitId || _load_bitId()).BitIds();
    _this.flattenedDevDependencies = flattenedDevDependencies || new (_bitId || _load_bitId()).BitIds();
    _this.packageDependencies = packageDependencies || {};
    _this.devPackageDependencies = devPackageDependencies || {};
    _this.bindingPrefix = bindingPrefix;
    _this.validateVersion();
    return _this;
  }

  _createClass(Version, [{
    key: 'validateVersion',
    value: function validateVersion() {
      var _this2 = this;

      var nonEmptyFields = ['mainFile', 'files'];
      nonEmptyFields.forEach(function (field) {
        if (!_this2[field]) throw new Error(`failed creating a version object, the field "${field}" can't be empty`);
      });
    }
  }, {
    key: 'id',
    value: function id() {
      var obj = this.toObject();

      // remove importSpecifiers from the ID, it's not needed for the ID calculation.
      // @todo: remove the entire dependencies.relativePaths from the ID (it's going to be a breaking change)

      var getDependencies = function getDependencies(deps) {
        var dependencies = (_ramda || _load_ramda()).default.clone(deps);
        if (dependencies && dependencies.length) {
          dependencies.forEach(function (dependency) {
            if (dependency.relativePaths && dependency.relativePaths.length) {
              dependency.relativePaths.forEach(function (relativePath) {
                if (relativePath.importSpecifiers) delete relativePath.importSpecifiers;
              });
            }
          });
        }
        return dependencies;
      };

      var filterFunction = function filterFunction(val, key) {
        if (key === 'devDependencies' || key === 'devPackageDependencies') return !(_ramda || _load_ramda()).default.isEmpty(val);
        return !!val;
      };

      return JSON.stringify((0, (_utils || _load_utils()).filterObject)({
        mainFile: obj.mainFile,
        files: obj.files,
        compiler: obj.compiler,
        tester: obj.tester,
        log: obj.log,
        dependencies: getDependencies(obj.dependencies),
        devDependencies: getDependencies(obj.devDependencies),
        packageDependencies: obj.packageDependencies,
        devPackageDependencies: obj.devPackageDependencies,
        bindingPrefix: obj.bindingPrefix
      }, filterFunction));
    }
  }, {
    key: 'getAllFlattenedDependencies',
    value: function getAllFlattenedDependencies() {
      return this.flattenedDependencies.concat(this.flattenedDevDependencies);
    }
  }, {
    key: 'getAllDependencies',
    value: function getAllDependencies() {
      return this.dependencies.dependencies.concat(this.devDependencies.dependencies);
    }
  }, {
    key: 'collectDependencies',
    value: function collectDependencies(scope, withEnvironments) {
      var dev = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      var envDependencies = [this.compiler, this.tester];
      var flattenedDependencies = dev ? this.flattenedDevDependencies : this.flattenedDependencies;
      var dependencies = withEnvironments ? flattenedDependencies.concat(envDependencies) : flattenedDependencies;
      var allDependencies = dependencies.concat(this.flattenedDevDependencies);
      return scope.importManyOnes(allDependencies, true);
    }
  }, {
    key: 'refs',
    value: function refs() {
      var files = this.files ? this.files.map(function (file) {
        return file.file;
      }) : [];
      var dists = this.dists ? this.dists.map(function (dist) {
        return dist.file;
      }) : [];
      return [].concat(_toConsumableArray(dists), _toConsumableArray(files)).filter(function (ref) {
        return ref;
      });
    }
  }, {
    key: 'toObject',
    value: function toObject() {
      return (0, (_utils || _load_utils()).filterObject)({
        files: this.files ? this.files.map(function (file) {
          return {
            file: file.file.toString(),
            relativePath: file.relativePath,
            name: file.name,
            test: file.test
          };
        }) : null,
        mainFile: this.mainFile,
        dists: this.dists ? this.dists.map(function (dist) {
          return {
            file: dist.file.toString(),
            relativePath: dist.relativePath,
            name: dist.name,
            test: dist.test
          };
        }) : null,
        compiler: this.compiler ? this.compiler.toString() : null,
        bindingPrefix: this.bindingPrefix || (_constants || _load_constants()).DEFAULT_BINDINGS_PREFIX,
        tester: this.tester ? this.tester.toString() : null,
        log: {
          message: this.log.message,
          date: this.log.date,
          username: this.log.username,
          email: this.log.email
        },
        ci: this.ci,
        specsResults: this.specsResults,
        docs: this.docs,
        dependencies: this.dependencies.cloneAsString(),
        devDependencies: this.devDependencies.cloneAsString(),
        flattenedDependencies: this.flattenedDependencies.map(function (dep) {
          return dep.toString();
        }),
        flattenedDevDependencies: this.flattenedDevDependencies.map(function (dep) {
          return dep.toString();
        }),
        packageDependencies: this.packageDependencies,
        devPackageDependencies: this.devPackageDependencies
      }, function (val) {
        return !!val;
      });
    }
  }, {
    key: 'toBuffer',
    value: function toBuffer(pretty) {
      var obj = this.toObject();
      var args = (0, (_utils || _load_utils()).getStringifyArgs)(pretty);
      var str = JSON.stringify.apply(JSON, [obj].concat(_toConsumableArray(args)));
      return (0, (_utils || _load_utils()).bufferFrom)(str);
    }
  }, {
    key: 'setSpecsResults',
    value: function setSpecsResults(specsResults) {
      this.specsResults = specsResults;
    }
  }, {
    key: 'setDist',
    value: function setDist(dist) {
      this.dist = dist ? {
        file: dist.hash(),
        name: (_constants || _load_constants()).DEFAULT_BUNDLE_FILENAME
      } : null;
    }
  }, {
    key: 'setCIProps',
    value: function setCIProps(ci) {
      this.ci = ci;
    }
  }], [{
    key: 'parse',
    value: function parse(contents) {
      var _JSON$parse = JSON.parse(contents),
          mainFile = _JSON$parse.mainFile,
          dists = _JSON$parse.dists,
          files = _JSON$parse.files,
          compiler = _JSON$parse.compiler,
          bindingPrefix = _JSON$parse.bindingPrefix,
          tester = _JSON$parse.tester,
          log = _JSON$parse.log,
          docs = _JSON$parse.docs,
          ci = _JSON$parse.ci,
          specsResults = _JSON$parse.specsResults,
          dependencies = _JSON$parse.dependencies,
          flattenedDependencies = _JSON$parse.flattenedDependencies,
          devDependencies = _JSON$parse.devDependencies,
          flattenedDevDependencies = _JSON$parse.flattenedDevDependencies,
          devPackageDependencies = _JSON$parse.devPackageDependencies,
          packageDependencies = _JSON$parse.packageDependencies;

      var getDependencies = function getDependencies() {
        var deps = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

        if (deps.length && (_ramda || _load_ramda()).default.is(String, (0, (_utils || _load_utils()).first)(deps))) {
          // backward compatibility
          return deps.map(function (dependency) {
            return { id: (_bitId || _load_bitId()).BitId.parse(dependency) };
          });
        }

        return deps.map(function (dependency) {
          return {
            id: (_bitId || _load_bitId()).BitId.parse(dependency.id),
            relativePaths: dependency.relativePaths
          };
        });
      };

      return new Version({
        mainFile,
        files: files ? files.map(function (file) {
          return { file: (_objects || _load_objects()).Ref.from(file.file), relativePath: file.relativePath, name: file.name, test: file.test };
        }) : null,
        dists: dists ? dists.map(function (dist) {
          return { file: (_objects || _load_objects()).Ref.from(dist.file), relativePath: dist.relativePath, name: dist.name, test: dist.test };
        }) : null,
        compiler: compiler ? (_bitId || _load_bitId()).BitId.parse(compiler) : null,
        bindingPrefix: bindingPrefix || null,
        tester: tester ? (_bitId || _load_bitId()).BitId.parse(tester) : null,
        log: {
          message: log.message,
          date: log.date,
          username: log.username,
          email: log.email
        },
        ci,
        specsResults,
        docs,
        dependencies: getDependencies(dependencies),
        flattenedDependencies: (_bitId || _load_bitId()).BitIds.deserialize(flattenedDependencies),
        devDependencies: getDependencies(devDependencies),
        flattenedDevDependencies: (_bitId || _load_bitId()).BitIds.deserialize(flattenedDevDependencies),
        devPackageDependencies,
        packageDependencies
      });
    }
  }, {
    key: 'from',
    value: function from(versionProps) {
      return new Version(versionProps);
    }
  }, {
    key: 'fromComponent',
    value: function fromComponent(_ref2) {
      var component = _ref2.component,
          files = _ref2.files,
          dists = _ref2.dists,
          flattenedDependencies = _ref2.flattenedDependencies,
          flattenedDevDependencies = _ref2.flattenedDevDependencies,
          message = _ref2.message,
          specsResults = _ref2.specsResults,
          username = _ref2.username,
          email = _ref2.email;

      return new Version({
        mainFile: component.mainFile,
        files: files ? files.map(function (file) {
          return { file: file.file.hash(), relativePath: file.relativePath, name: file.name, test: file.test };
        }) : null,
        dists: dists ? dists.map(function (dist) {
          return { file: dist.file.hash(), relativePath: dist.relativePath, name: dist.name, test: dist.test };
        }) : null,
        compiler: component.compilerId,
        bindingPrefix: component.bindingPrefix,
        tester: component.testerId,
        log: {
          message,
          username,
          email,
          date: Date.now().toString()
        },
        specsResults,
        docs: component.docs,
        packageDependencies: component.packageDependencies,
        devPackageDependencies: component.devPackageDependencies,
        flattenedDependencies,
        flattenedDevDependencies,
        dependencies: component.dependencies.get(),
        devDependencies: component.devDependencies.get()
      });
    }
  }]);

  return Version;
}((_objects || _load_objects()).BitObject);

exports.default = Version;
//# sourceMappingURL=version.js.map