'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _child_process;

function _load_child_process() {
  return _child_process = require('child_process');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope/scope'));
}

var _specsResults;

function _load_specsResults() {
  return _specsResults = require('../consumer/specs-results');
}

var _abstractVinyl;

function _load_abstractVinyl() {
  return _abstractVinyl = _interopRequireDefault(require('../consumer/component/sources/abstract-vinyl'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function run(_ref) {
  var testerFilePath = _ref.testerFilePath,
      testerId = _ref.testerId,
      mainFile = _ref.mainFile,
      testFile = _ref.testFile;

  function getDebugPort() {
    var debugPortArgName = '--debug-brk';
    try {
      var execArgv = process.execArgv.map(function (arg) {
        return arg.split('=');
      });
      var execArgvObj = (_ramda || _load_ramda()).default.fromPairs(execArgv);
      if (execArgvObj[debugPortArgName]) return parseInt(execArgvObj[debugPortArgName]);
    } catch (e) {
      return null;
    }

    return null;
  }

  return new Promise(function (resolve, reject) {
    var debugPort = getDebugPort();
    var openPort = debugPort ? debugPort + 1 : null;

    var child = (0, (_child_process || _load_child_process()).fork)((_path || _load_path()).default.join(__dirname, 'worker.js'), {
      execArgv: openPort ? [`--debug=${openPort.toString()}`] : [],
      silent: false,
      env: {
        __mainFile__: mainFile,
        __testFilePath__: testFile.path,
        __tester__: testerFilePath,
        __testerId__: testerId.toString()
      }
    });

    process.on('exit', function () {
      child.kill('SIGKILL');
    });

    child.on('message', function (_ref2) {
      var type = _ref2.type,
          payload = _ref2.payload;

      if (type === 'error') return reject(payload);
      if (payload.specPath) payload.specPath = testFile.relative;
      return resolve(payload);
    });

    child.on('error', function (e) {
      reject(e);
    });
  });
}

exports.default = {
  run
};
//# sourceMappingURL=specs-runner.js.map