'use strict';

// TODO - move to language specific driver.

var serializeError = require('serialize-error');

try {
  var mainFilePath = process.env.__mainFile__;
  var testFilePath = process.env.__testFilePath__;
  var testerFilePath = process.env.__tester__;

  var mockery = require('mockery');
  mockery.enable({
    warnOnReplace: false,
    warnOnUnregistered: false,
    useCleanCache: true
  }); // enable mocks on process

  var tester = require(testerFilePath);

  // define the __impl__ global
  global.__impl__ = mainFilePath;

  // register globals
  if (tester.globals) {
    Object.keys(tester.globals).forEach(function (g) {
      global[g] = tester.globals[g];
    });
  }

  // register modules
  if (tester.modules) {
    Object.keys(tester.modules).forEach(function (m) {
      mockery.registerMock(m, tester.modules[m]);
    });
  }

  if (!tester.run) {
    process.send({
      type: 'error',
      payload: `"${process.env.__testerId__}" doesn't have a valid tester interface`
    });
    process.exit(1);
  }

  tester.run(testFilePath).then(function (results) {
    if (!results) throw new Error(`tester did not return any result for the file ${testFilePath}`);
    mockery.disable();
    results.specPath = testFilePath;
    return process.send({ type: 'results', payload: results });
  }).catch(function (err) {
    mockery.disable();
    return process.send({ type: 'error', payload: serializeError(err) });
  });
} catch (e) {
  process.send({ type: 'error', payload: serializeError(e) });
}
//# sourceMappingURL=worker.js.map