'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pathHas = pathHas;
exports.propogateUntil = propogateUntil;

var _path;

function _load_path() {
  return _path = _interopRequireWildcard(require('path'));
}

var _fs;

function _load_fs() {
  return _fs = _interopRequireWildcard(require('fs'));
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function composePath(patternPath, patterns) {
  return patterns.map(function (pattern) {
    return (_path || _load_path()).join(patternPath, pattern);
  });
}

/**
 * determine whether given path matches patterns
 */
function pathHas(patterns) {
  return function (absPath) {
    var state = false;
    var paths = composePath(absPath, patterns);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = paths[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var potentialPath = _step.value;

        if (state) return state;
        state = (_fs || _load_fs()).existsSync(potentialPath);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return state;
  };
}

/**
 * propogates the FS from given path and until passing pattern function test.
 * @name propogateUntil
 * @param {string} fromPath
 * @param {(path: string) => boolean} pattern
 * @returns {string|null} first path to pass the test.
 * @example
 * ```js
 *  propogateUntil('/usr/local/var', (path) => path.indexOf('/usr') !== -1);
 *  // => '/usr/local/var'
 * ```
 */
function propogateUntil(fromPath, pattern) {
  function buildPropogationPaths() {
    var paths = [];
    var pathParts = fromPath.split((_path || _load_path()).sep);

    pathParts.forEach(function (val, index) {
      var part = pathParts.slice(0, index + 1).join('/');
      if (!part) return;
      paths.push(part);
    });

    return paths.reverse();
  }

  if (pattern(fromPath)) return fromPath;
  var searchPaths = buildPropogationPaths();
  return searchPaths.find(function (searchPath) {
    return pattern(searchPath);
  });
}
//# sourceMappingURL=propogate-until.js.map