/** @flow */
import fs from 'fs-extra';
import { AUTO_GENERATED_STAMP, AUTO_GENERATED_MSG } from '../constants';

export default function outputFile({
  filePath,
  content,
  writeAutoGeneratedMessage = true,
  override = true
}: {
  filePath: string,
  content: string,
  writeAutoGeneratedMessage?: boolean,
  override?: boolean
}) {
  return new Promise((resolve, reject) => {
    if (!override && fs.existsSync(filePath)) {
      const fileContent = fs.readFileSync(filePath).toString();
      if (!fileContent.includes(AUTO_GENERATED_STAMP)) return resolve(filePath);
    }
    const data = writeAutoGeneratedMessage ? AUTO_GENERATED_MSG + content : content;
    fs.outputFile(filePath, data, (err) => {
      if (err) return reject(err);
      return resolve(filePath);
    });
  });
}
